@@ -1,7 +1,14 @@
-import pkg_resources
+import sys
+
 import numba
+
 from .pynndescent_ import NNDescent, PyNNDescentTransformer
 
+if sys.version_info[:2] >= (3, 8):
+    import importlib.metadata as importlib_metadata
+else:
+    import importlib_metadata
+
 # Workaround: https://github.com/numba/numba/issues/3341
 if numba.config.THREADING_LAYER == "omp":
     try:
@@ -12,4 +19,4 @@ if numba.config.THREADING_LAYER == "omp":
         # might be a missing symbol due to e.g. tbb libraries missing
         numba.config.THREADING_LAYER = "workqueue"
 
-__version__ = pkg_resources.get_distribution("pynndescent").version
+__version__ = importlib_metadata.version("pynndescent")
